@@ -95,6 +95,7 @@ class Config(piupartslib.conf.Config):
                                          "chroot-tgz": None,
                                          "upgrade-test-distros": None,
                                          "basetgz-directory": ".",
+                                         "chroot-meta-auto": None,
                                          "max-reserved": 1,
                                          "debug": "no",
                                          "keep-sources-list": "no",
@@ -640,6 +641,15 @@ class Section:
         self._check_tarball()
         if not os.path.exists(self._get_tarball()):
             self._error_wait_until = time.time() + 300
+        if self._config["chroot-meta-auto"]:
+            if os.path.exists(self._config["chroot-meta-auto"]):
+                try:
+                    age = time.time() - os.path.getmtime(self._config["chroot-meta-auto"])
+                    if age > 6 * 3600:
+                        os.unlink(self._config["chroot-meta-auto"])
+                        logging.info("Deleting old %s" % self._config["chroot-meta-auto"])
+                except OSError:
+                    pass
         for package_name, version in self._slave.get_reserved():
             self._throttle_if_overloaded()
             if interrupted or got_sighup:
@@ -690,6 +700,11 @@ class Section:
                 command.extend(["-d", distro])
         if self._config["keep-sources-list"] in ["yes", "true"]:
             command.append("--keep-sources-list")
+        if distupgrade and self._config["chroot-meta-auto"]:
+            if not os.path.exists(self._config["chroot-meta-auto"]):
+                command.extend(["-S", self._config["chroot-meta-auto"]])
+            else:
+                command.extend(["-B", self._config["chroot-meta-auto"]])
         command.extend(["--apt", "%s=%s" % (pname, pvers)])
 
         subdir = "fail"
@@ -755,7 +770,7 @@ class Section:
             subdir = "untestable"
 
         if ret == 0:
-            output.write("Executing: %s\n" % " ".join(quote_spaces(command)))
+            output.write("Executing: %s\n" % command2string(command))
             ret, f = run_test_with_timeout(command, MAX_WAIT_TEST_RUN)
             if not f or f[-1] != '\n':
                 f += '\n'
@@ -766,6 +781,13 @@ class Section:
             elif not "piuparts run ends" in lastline:
                 ret += 1024
                 output.write(" *** PIUPARTS OUTPUT INCOMPLETE ***\n")
+            elif distupgrade and self._config["chroot-meta-auto"] and \
+                    "History of available packages does not match - reference chroot may be outdated" in f:
+                try:
+                    os.unlink(self._config["chroot-meta-auto"])
+                    logging.info("Deleting outdated %s" % self._config["chroot-meta-auto"])
+                except OSError:
+                    pass
 
         output.write("\n")
         output.write("ret=%d\n" % ret)
@@ -783,8 +805,9 @@ def log_name(package, version):
     return "%s_%s.log" % (package, version)
 
 
-def quote_spaces(vlist):
-    return [pipes.quote(x) for x in vlist]
+def command2string(command):
+    """Quote s.t. copy+paste from the logfile gives a runnable command in the shell."""
+    return " ".join([pipes.quote(arg) for arg in command])
 
 
 def run_test_with_timeout(cmd, maxwait, kill_all=True):
@@ -826,7 +849,7 @@ def run_test_with_timeout(cmd, maxwait, kill_all=True):
                 except OSError:
                     pass
 
-    logging.debug("Executing: %s" % " ".join(quote_spaces(cmd)))
+    logging.debug("Executing: %s" % command2string(cmd))
 
     stdout = ""
     p = subprocess.Popen(cmd, preexec_fn=os.setpgrp,
@@ -872,7 +895,8 @@ def create_chroot(config, tarball, distro):
     command.extend(["--arch", config.get_arch()])
     command.extend(["-d", distro])
     command.extend(["-s", tarball + ".new"])
-    command.extend(["--apt", "dpkg"])
+    command.extend(['--no-install-purge-test', '--no-upgrade-test'])
+    command.extend(["--apt", "TARBALL"])  # dummy package name
 
     output_name = tarball + ".log"
     with open(output_name, "we") as output:
@@ -884,8 +908,8 @@ def create_chroot(config, tarball, distro):
             logging.info("Creating new tarball %s" % tarball)
             output.write(time.strftime("Start: %Y-%m-%d %H:%M:%S %Z\n\n",
                                        time.gmtime()))
-            output.write("Executing: " + " ".join(quote_spaces(command)) + "\n\n")
-            logging.debug("Executing: " + " ".join(quote_spaces(command)))
+            output.write("Executing: " + command2string(command) + "\n\n")
+            logging.debug("Executing: " + command2string(command))
             p = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
             for line in p.stdout:
                 output.write(line)
@@ -937,6 +961,10 @@ def main():
         logging.error("no sections found")
         return
 
+    # flush logs from previous run
+    for section in sections:
+        section.run(do_processing=False)
+
     while True:
         global got_sighup
         test_count = 0
