@@ -276,6 +276,7 @@ class PackagesDB:
         "unknown",
         "unknown-preferred-alternative",  # obsolete
         "no-dependency-from-alternatives-exists",  # obsolete
+        "outdated",
         # "foreign:*",  # can only happen as query result for a dependency
         # "does-not-exist",  # can only happen as query result for a dependency
     ]
@@ -446,7 +447,13 @@ class PackagesDB:
                 return True
         return False
 
-    def _lookup_package_state(self, package, use_cached_success):
+    def _lookup_package_state(self, package, use_cached_success, check_outdated):
+        if check_outdated:
+            # Check if dependency databases have a newer version of this package.
+            for db in self._dependency_databases:
+                v = db.get_version(package["Package"])
+                if v is not None and apt_pkg.version_compare(package["Version"], v) < 0:
+                    return "outdated";
         if self._recycle_mode and self._logdb.log_exists(package, [self._recycle]):
             return "unknown"
         if self._logdb.log_exists(package, [self._ok]):
@@ -555,15 +562,7 @@ class PackagesDB:
 
         return "unknown"
 
-    def _compute_package_states(self, use_cached_success=False):
-        if self._in_state is not None:
-            return
-
-        self._stamp = time.time()
-
-        for subdir in self._all:
-            self._logdb.bulk_load_dir(subdir)
-
+    def _initialize_package_states(self, use_cached_success, check_outdated):
         self._find_all_packages()
 
         self._package_state = {}
@@ -573,7 +572,7 @@ class PackagesDB:
         todo = []
 
         for package_name, package in self._packages.iteritems():
-            state = self._lookup_package_state(package, use_cached_success)
+            state = self._lookup_package_state(package, use_cached_success, check_outdated)
             assert state in self._states
             self._package_state[package_name] = state
             if state == "unknown":
@@ -581,9 +580,26 @@ class PackagesDB:
             else:
                 self._in_state[state].append(package_name)
 
+        return todo
+
+    def _compute_package_states(self, use_cached_success=False):
+        if self._in_state is not None:
+            return
+
+        self._stamp = time.time()
+
+        for subdir in self._all:
+            self._logdb.bulk_load_dir(subdir)
+
+        todo = self._initialize_package_states(use_cached_success=use_cached_success, check_outdated=False)
+
         for db in self._dependency_databases:
             db._compute_package_states(use_cached_success=True)
 
+        if self._dependency_databases:
+            # redo the initialization to properly resolve "outdated" packages after the dependency databases have been initialized
+            todo = self._initialize_package_states(use_cached_success=use_cached_success, check_outdated=True)
+
         while todo:
             package_names = todo
             todo = []
@@ -597,7 +613,7 @@ class PackagesDB:
                     else:
                         self._in_state[state].append(package_name)
                         self._package_state[package_name] = state
-                        done.append(package)
+                        done.append(package_name)
             if not done:
                 # If we didn't do anything this time, we sure aren't going
                 # to do anything the next time either.
@@ -702,6 +718,11 @@ class PackagesDB:
     def get_package_state(self, package_name, resolve_virtual=True, recurse=True):
         self._compute_package_states()
         if package_name in self._package_state:
+            if recurse and self._package_state[package_name] == "outdated":
+                for db in self._dependency_databases:
+                    state = db.get_package_state(package_name, resolve_virtual=resolve_virtual, recurse=False)
+                    if state not in ["does-not-exist", "outdated"]:
+                        return state
             return self._package_state[package_name]
         if package_name in self._virtual_packages:
             if resolve_virtual:
